/*
 * Needlesoft Haystack Explorer
 * Ludumdare 16 entry
 * Made in about 6 hours
 */

#include <stdio.h>
#include <allegro.h>
#include <alleggl.h>
#include <math.h>
#include <SOIL.h>
#include <fmodex/fmod.h>


// Fine tuning defines
// "trees" are actually haystraws, and yes, there are 100 000 of these
#define TURN_SPEED .1
#define MOVE_SPEED 1.0
#define NUM_TREES  100000
#define WORLD_SIZE 200.0
#define HAYSTACK 100.0

// Macros
#define FRAND(x) (((rand()%(int)x)*1024.0)/1024.0)
#define DEG(n)   ((n) * 180.0 / M_PI)
//#define SGN(n)  (n>=0 ? 1 : -1)

// Player states
#define WALK 1
#define IDLE 2
#define STOP 3

// World properties
#define HEIGHT 10.0
#define HEIGHTF 40.0
#define MAXV 2048
#define NUM_CLOUDS 20
#define CURSOR_SIZE .04

// Vertex and colour arrays
float tree[NUM_TREES][2][3],colors[NUM_TREES][3],clouds[NUM_CLOUDS][6][3],needle[2][6];

// state variables
int frames = 0,state,recolour=0,gameover,intro=1;

// timer var
volatile int tim;


// player (camera) position
float playerx,playery,playerz,lookx,looky,lookz;

// intro timer
float introroll;

// mouse to world transform arrays
float final_point[3];
float point_xformed[3];
float lookf;

// filename temporary string
char temp[256];

// textures
GLuint cloud,fence,introtex,outro;

// texture temporary bitmap
BITMAP *mybmp;

// music
FMOD_SYSTEM      *soundsystem;
FMOD_SOUND       *theme,*intromus,*outromus;
FMOD_CHANNEL     *channel = 0;

// checks if ray cast from mouse reached the needle
void seek()
{
  float d,x,y,z;
  x=final_point[0]-((needle[0][0]+needle[1][0])/2);
  y=final_point[1]-((needle[0][1]+needle[1][1])/2);
  z=final_point[2]-((needle[0][2]+needle[1][2])/2);
  d=sqrt(x*x+y*y+z*z);
   if (d<10.0) {
     gameover=1;

      FMOD_Sound_Release(theme);
      FMOD_System_PlaySound(soundsystem, FMOD_CHANNEL_FREE, outromus, 0, &channel);
    }
}

// mouse raycast algorithm (stolen, source: http://www.idevgames.com/forum/showthread.php?t=16515 )
void grab_mouse( int x, int y )
{
    float value_fov =(M_PI/2);
    float value_aspect = 1.3333;
    float half_window_width = 320.0;
    float half_window_height = 240.0;
    float modifier_x;
    float modifier_y;
    float point[3];
    float point_dist = 10.0;
    float camera_origin[3];
    GLfloat pulledMatrix[16];
    float color[4] = { 0.0, 1.0, 0.0, 1.0 };
    modifier_x = tan( value_fov * 0.5 )
        * (( 1.0 - x / half_window_width ) * ( value_aspect ) );
    modifier_y = tan( value_fov * 0.5 )
        * -( 1.0 - y / half_window_height );
    point[0] = modifier_x * point_dist;
    point[1] = modifier_y * point_dist;
    point[2] = point_dist;
    glGetFloatv(GL_MODELVIEW_MATRIX, pulledMatrix);
    camera_origin[0] = -(
        pulledMatrix[0] * pulledMatrix[12] +
        pulledMatrix[1] * pulledMatrix[13] +
        pulledMatrix[2] * pulledMatrix[14]);
    camera_origin[1] = -(
        pulledMatrix[4] * pulledMatrix[12] +
        pulledMatrix[5] * pulledMatrix[13] +
        pulledMatrix[6] * pulledMatrix[14]);
    camera_origin[2] = -(
        pulledMatrix[8] * pulledMatrix[12] +
        pulledMatrix[9] * pulledMatrix[13] +
        pulledMatrix[10] * pulledMatrix[14]);
    point_xformed[0] = -(
        pulledMatrix[0] * point[0] +
        pulledMatrix[1] * point[1] +
        pulledMatrix[2] * point[2]);
    point_xformed[1] = -(
        pulledMatrix[4] * point[0] +
        pulledMatrix[5] * point[1] +
        pulledMatrix[6] * point[2]);
    point_xformed[2] = -(
        pulledMatrix[8] * point[0] +
        pulledMatrix[9] * point[1] +
        pulledMatrix[10] * point[2]);
    final_point[0] = (point_xformed[0]) + camera_origin[0];
    final_point[1] = (point_xformed[1]) + camera_origin[1];
    final_point[2] = (point_xformed[2]) + camera_origin[2];

}

// draws hay (they supposed to be trees, but i had to change the concept)
void draw_tree()
{
  int c,b,n;

// commented out slow direct-mode drawing

  //glPopMatrix();
  //glPushMatrix();
  //glTranslatef(tree[n][0],0,tree[n][1]);
  /*
  glBegin(GL_QUADS)
  glBegin(GL_LINES);
    for (c=0;c<NUM_TREES;c++)
    {
      glColor3f(0.0,0.0,0.0);
      glVertex3fv(&tree[c][0]);
      glColor3f(1.0,.8,.0);
      glVertex3fv(&tree[c][1]);
    }
  glEnd();
  */

// draws ground quad
  glColor3f(.8,.7,.1);
  glBegin(GL_QUADS);
     glVertex3f(0,0.0,0);
     glVertex3f(0,0.0,WORLD_SIZE);
     glVertex3f(WORLD_SIZE,0.0,WORLD_SIZE);
     glVertex3f(WORLD_SIZE,0.0,0);
  glEnd();

// cloud "hanger wires"
  glColor3f(.0,.0,.0);
  glBegin(GL_LINES);
  for (c=0;c<NUM_CLOUDS;c++)
  {
     glVertex3fv((GLfloat*)&clouds[c][4]);
     glVertex3fv((GLfloat*)&clouds[c][5]);
  }
  glEnd();

// needle
  if (!intro && !gameover)
  {
  glColor3f(.0,.0,1.0);
  glBegin(GL_LINES);
     glVertex3fv((GLfloat*)&needle[0]);
     glVertex3fv((GLfloat*)&needle[1]);
  glEnd();
  }

// draws clouds
  glColor3f(1.0,1.0,1.0);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,cloud);
  glBegin(GL_QUADS);
  for (c=0;c<NUM_CLOUDS;c++)
  {
     glTexCoord2f(0.0,0.0);
     glVertex3fv((GLfloat*)&clouds[c][0]);
     glTexCoord2f(1.0,0.0);
     glVertex3fv((GLfloat*)&clouds[c][1]);
     glTexCoord2f(1.0,1.0);
     glVertex3fv((GLfloat*)&clouds[c][2]);
     glTexCoord2f(0.0,1.0);
     glVertex3fv((GLfloat*)&clouds[c][3]);
  }
  glEnd();

// fence
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,fence);
  glBegin(GL_QUADS);
     glTexCoord2f(0.0,0.0);
     glVertex3f(0,0.0,0);
     glTexCoord2f(1.0,0.0);
     glVertex3f(0,0.0,WORLD_SIZE);
     glTexCoord2f(1.0,1.0);
     glVertex3f(0,HEIGHTF,WORLD_SIZE);
     glTexCoord2f(0.0,1.0);
     glVertex3f(0,HEIGHTF,0);

     glTexCoord2f(0.0,0.0);
     glVertex3f(0,0.0,WORLD_SIZE);
     glTexCoord2f(1.0,0.0);
     glVertex3f(WORLD_SIZE,0.0,WORLD_SIZE);
     glTexCoord2f(1.0,1.0);
     glVertex3f(WORLD_SIZE,HEIGHTF,WORLD_SIZE);
     glTexCoord2f(0.0,1.0);
     glVertex3f(0,HEIGHTF,WORLD_SIZE);

     glTexCoord2f(0.0,0.0);
     glVertex3f(WORLD_SIZE,0.0,WORLD_SIZE);
     glTexCoord2f(1.0,0.0);
     glVertex3f(WORLD_SIZE,0.0,0);
     glTexCoord2f(1.0,1.0);
     glVertex3f(WORLD_SIZE,HEIGHTF,0);
     glTexCoord2f(0.0,1.0);
     glVertex3f(WORLD_SIZE,HEIGHTF,WORLD_SIZE);

     glTexCoord2f(0.0,0.0);
     glVertex3f(WORLD_SIZE,0.0,0);
     glTexCoord2f(1.0,0.0);
     glVertex3f(0,0.0,0);
     glTexCoord2f(1.0,1.0);
     glVertex3f(0,HEIGHTF,0);
     glTexCoord2f(0.0,1.0);
     glVertex3f(WORLD_SIZE,HEIGHTF,0);

  glEnd();

// haystack
  glDisable(GL_TEXTURE_2D);
  glLineWidth(5);
  glDrawArrays(GL_LINES, 0,NUM_TREES);

}


// generates haystack, clouds and needle poasition and properties
void generate_stuff()
{
  int c;

  for (c=0;c<NUM_TREES;c++)
  {

  // needles are lines, have 2 points tree[c][0] and tree[c][1] each has 3 coordinates

  // generates needle position (first vertex)
  tree[c][0][0]=FRAND(WORLD_SIZE);
  tree[c][0][2]=FRAND(WORLD_SIZE);
  tree[c][0][1]=FRAND(HAYSTACK*sin((tree[c][0][0]/WORLD_SIZE)*M_PI)*sin((tree[c][0][2]/WORLD_SIZE)*M_PI)*sin((tree[c][0][0]/WORLD_SIZE)*M_PI)*sin((tree[c][0][2]/WORLD_SIZE)*M_PI));

  // generates second point nearby the first
  tree[c][1][0]=tree[c][0][0]-10.0+FRAND(20.0);
  tree[c][1][1]=tree[c][0][1]-10.0+FRAND(20.0);
  tree[c][1][2]=tree[c][0][2]-10.0+FRAND(20.0);

  //randomizing colour for vertices
  colors[c][0]=.2+(FRAND(800.0)/1000.0);
  colors[c][1]=colors[c][0]/(1.2+(FRAND(200.0)/1000.0));
  colors[c][2]=(FRAND(200.0)/1000.0);
  }

  // this was supposed to gererate needle position inside the haystack
  // too hard too find, commented out
  /*
  needle[0][0]=FRAND(WORLD_SIZE);
  needle[0][2]=FRAND(WORLD_SIZE);
  needle[0][1]=FRAND(HAYSTACK*sin((needle[0][0]/WORLD_SIZE)*M_PI)*sin((needle[0][2]/WORLD_SIZE)*M_PI)*sin((needle[0][0]/WORLD_SIZE)*M_PI)*sin((needle[0][2]/WORLD_SIZE)*M_PI));
  needle[1][0]=needle[0][0]-10.0+FRAND(20.0);
  needle[1][1]=needle[0][1]-10.0+FRAND(20.0);
  needle[1][2]=needle[0][2]-10.0+FRAND(20.0);
  */

  // this generates needle on haystack surface
  needle[0][0]=FRAND(WORLD_SIZE);
  needle[0][2]=FRAND(WORLD_SIZE);
  needle[0][1]=5.0+HAYSTACK*sin((needle[0][0]/WORLD_SIZE)*M_PI)*sin((needle[0][2]/WORLD_SIZE)*M_PI)*sin((needle[0][0]/WORLD_SIZE)*M_PI)*sin((needle[0][2]/WORLD_SIZE)*M_PI);
  needle[1][0]=needle[0][0]-10.0+FRAND(20.0);
  needle[1][1]=needle[0][1]-10.0+FRAND(20.0);
  needle[1][2]=needle[0][2]-10.0+FRAND(20.0);

  // generates cloud vertices
  // first four (0-3) are cloud quad coordinates
  // next two (4-5) are for cloud "hangers"
  for (c=0;c<NUM_CLOUDS;c++)
  {
    clouds[c][0][0]=(3*FRAND(WORLD_SIZE))-WORLD_SIZE;
    clouds[c][0][1]=(HAYSTACK*1.8)+FRAND(100.0);
    clouds[c][0][2]=(3*FRAND(WORLD_SIZE))-WORLD_SIZE;
    clouds[c][1][0]=clouds[c][0][0]+80.0+FRAND(100.0);
    clouds[c][1][1]=clouds[c][0][1];
    clouds[c][1][2]=clouds[c][0][2]+80.0+FRAND(100.0);
    clouds[c][2][0]=clouds[c][1][0];
    clouds[c][2][1]=clouds[c][1][1]-40.0-FRAND(80.0);
    clouds[c][2][2]=clouds[c][1][2];
    clouds[c][3][0]=clouds[c][0][0];
    clouds[c][3][1]=clouds[c][2][1];
    clouds[c][3][2]=clouds[c][0][2];
    clouds[c][4][0]=(clouds[c][0][0]+clouds[c][1][0])/2;
    clouds[c][4][1]=((clouds[c][0][1]+clouds[c][2][1])/2)-4.0;
    clouds[c][4][2]=(clouds[c][0][2]+clouds[c][1][2])/2;
    clouds[c][5][0]=clouds[c][4][0];
    clouds[c][5][1]=800.0;
    clouds[c][5][2]=clouds[c][4][2];

  }
}

// timed state updates
void timer_proc(void)
{
  float r,g,b;

  // if it's intro time, it spins camera around the haystack untile space is pressed
  if (intro)
  {
    introroll+=.02;
    playerx=(cos(introroll)*(WORLD_SIZE/2)+(WORLD_SIZE/2));
    playery=HAYSTACK;
    playerz=(sin(introroll)*(WORLD_SIZE/2)+(WORLD_SIZE/2));
    looky=introroll+(M_PI);
    lookx=M_PI/4;

    // stop music when intro ends, and changes state
    if (key[KEY_SPACE])
    {
       FMOD_Sound_Release(intromus);
      FMOD_System_PlaySound(soundsystem, FMOD_CHANNEL_FREE, theme, 0, &channel);
      intro=0;

    }
  } else {

  // actual game mechanics

  // turning left/right
	if (key[KEY_LEFT]) {looky -= TURN_SPEED;}
	if (key[KEY_RIGHT]) {looky += TURN_SPEED;}

  // moving; moves the player and change player state accordingly
	if (key[KEY_UP])
	{
	  playerx += cos(looky)*MOVE_SPEED;
	  playerz += sin(looky)*MOVE_SPEED;
	  state=WALK;
  }

	if (key[KEY_DOWN])
	{
	  playerx -= cos(looky)*MOVE_SPEED;
	  playerz -= sin(looky)*MOVE_SPEED;
	  state=WALK;
  }

  if (!key[KEY_UP] && !key[KEY_DOWN] && state!=IDLE)
    state=IDLE;


  // these three conditionals calculate vertical position according to the state
  // e.g. vertical camwera movement is different when walking and standing

  if (state==STOP)
  {
    if (ABS(playery-HEIGHT)>HAYSTACK*sin((playerx/WORLD_SIZE)*M_PI)*sin((playerx/WORLD_SIZE)*M_PI)*sin((playerz/WORLD_SIZE)*M_PI)*sin((playerz/WORLD_SIZE)*M_PI))
     {
       playery-=SGN(playery-HEIGHT)*.4;
     } else {
       state=IDLE;
       lookf=0.0;
     }
  }

  if (state==IDLE)
  {
    lookf+=.1;
    playery=((HEIGHT/10.0)*sin(lookf))+HEIGHT;
  }

  if (state==WALK)
  {
    lookf+=0.2;
    playery=HEIGHT+((HEIGHT/3.0)*sin(lookf));
  }

  // if inside the world, adds "heavy breath" wavy animation
  if (playerx> 0.0 && playerx<WORLD_SIZE && playerz> 0.0 && playerz<WORLD_SIZE)
  {
    playery+=(HAYSTACK*sin((playerx/WORLD_SIZE)*M_PI)*sin((playerx/WORLD_SIZE)*M_PI)*sin((playerz/WORLD_SIZE)*M_PI)*sin((playerz/WORLD_SIZE)*M_PI));
  } else playery+=0.0;

  // looks down on space
  if (key[KEY_SPACE])
  {
    if (lookx<M_PI/4) lookx+=.1;
  } else {
    if (lookx>0.0) lookx-=.2;
  }

  // keeps player inside the world
  if(playerx>(WORLD_SIZE-5.0)) playerx=(WORLD_SIZE-5.0);
  if(playerz>(WORLD_SIZE-5.0)) playerz=(WORLD_SIZE-5.0);
  if(playerx<5.0) playerx=5.0;
  if(playerz<5.0) playerz=5.0;

  // ray shot from mouse is actually a point and it moves forward
  // distance from this point to the needle is calculated each frame
  // this was done to save some time (and it works)
  final_point[0]+=point_xformed[0];
  final_point[1]+=point_xformed[1];
  final_point[2]+=point_xformed[2];

  // checks if "raypoint" caught the needle
  seek();
  //if (key[KEY_A]) gameover=1;
  }

}

void draw (void)
{
  int c;
  float r,g,b,mx,my;
  // if game is over, paints 100 haystraws green each frame until done with all of them
 if (gameover==1)
  {
    for (c=0;c<100;c++)
    {
    r=colors[recolour][0];
    g=colors[recolour][1];
    b=colors[recolour][2];
    colors[recolour][0]=b;
    colors[recolour][1]=r;
    colors[recolour][2]=(g/2);
    recolour++;
    // if all are painted, breaks from loop and changes game state
    if (recolour>=NUM_TREES) {gameover=2;c=110;}
    }

  }
  // standard matrix setup
  glMatrixMode (GL_PROJECTION);
	  glLoadIdentity ();
	  glPushMatrix();
	  glFrustum (-(4.0/3.0), 4.0/3.0, -1.0, 1.0, 1.0, 8000.0);
	glMatrixMode (GL_MODELVIEW);
	  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  glLoadIdentity();
	  glPushMatrix();
	  glRotatef (DEG(lookx), 1, 0, 0);
	  glRotatef (DEG(looky)+90.0, 0, 1, 0);
	  glRotatef (DEG(lookz), 0, 0, 1);
	  glTranslatef (-playerx, -playery, -playerz);

  // if mouse button pressed, shots the "raypoint" from mopuse position
  if (mouse_b)
  {
    grab_mouse(mouse_x,mouse_y);
   // printf("needle: %1.2f %1.2f %1.2f camera: %1.2f %1.2f %1.2f click: %1.2f %1.2f %1.2f\n",needle[0][0],needle[0][1],needle[0][2],playerx,playery,playerz,final_point[0],final_point[1],final_point[2]);
  }

  // draws the "raypoint"
  glColor3f(1.0,1.0,1.0);
  glBegin(GL_POINTS);
   glVertex3fv((GLfloat*)&final_point);
  glEnd( );

  // draws world (no trees, apparently :)
  draw_tree();

  // gets identity matrix back to draw HUD
  glMatrixMode (GL_PROJECTION);
    glPopMatrix();
  glMatrixMode (GL_MODELVIEW);
    glPopMatrix();

  glPointSize(3.0);
  glDisable(GL_TEXTURE_2D);

  // recalculates mouse screen position to match GL coordinate system
  glColor3f(1.0,1.0,1.0);
  mx=(mouse_x-320)/320.0;
  my=-(mouse_y-240)/240.0;

  // draws mouse cursor
  glBegin(GL_LINES);
   glColor4f(1.0,.0,.0,1.0);
    glVertex2f(mx-CURSOR_SIZE,my-CURSOR_SIZE);
   glColor4f(.0,1.0,.0,1.0);
    glVertex2f(mx+CURSOR_SIZE,my+CURSOR_SIZE);
   glColor4f(.0,.0,1.0,1.0);
    glVertex2f(mx+CURSOR_SIZE,my-CURSOR_SIZE);
   glColor4f(1.0,1.0,.0,1.0);
    glVertex2f(mx-CURSOR_SIZE,my+CURSOR_SIZE);
  glEnd();

  // if it's intro, it draws title screen texture
  if (intro)
  {
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,introtex);
  glBegin(GL_QUADS);
    glTexCoord2f(0.0,0.0);
    glVertex2f(-1.0,1.0);
    glTexCoord2f(1.0,0.0);
    glVertex2f(1.0,1.0);
    glTexCoord2f(1.0,1.0);
    glVertex2f(1.0,-1.0);
    glTexCoord2f(0.0,1.0);
    glVertex2f(-1.0,-1.0);
  glEnd();
  glDisable(GL_TEXTURE_2D);
  }
  // if it's game over, draws "you win" message
  if (gameover)
  {
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,outro);
  glBegin(GL_QUADS);
    glTexCoord2f(0.0,0.0);
    glVertex2f(-1.0,1.0);
    glTexCoord2f(1.0,0.0);
    glVertex2f(1.0,1.0);
    glTexCoord2f(1.0,1.0);
    glVertex2f(1.0,.5);
    glTexCoord2f(0.0,1.0);
    glVertex2f(-1.0,.5);
  glEnd();
  glDisable(GL_TEXTURE_2D);
  }

  // flip
	allegro_gl_flip();

}

// standard allegro timer
void timer(void)
{
	tim++;
}
END_OF_FUNCTION(timer);

int main ()
{
  int c;
	allegro_init();

  // inits allegro and allegrogl
	install_allegro_gl();
	allegro_gl_clear_settings();
	allegro_gl_set (AGL_COLOR_DEPTH, 32);
	allegro_gl_set (AGL_Z_DEPTH, 24);
	allegro_gl_set (AGL_WINDOWED, TRUE);
	allegro_gl_set (AGL_DOUBLEBUFFER, 1);
	allegro_gl_set (AGL_SUGGEST, AGL_COLOR_DEPTH | AGL_Z_DEPTH
	              | AGL_DOUBLEBUFFER | AGL_WINDOWED);

	if (set_gfx_mode(GFX_OPENGL, 640, 480, 0, 0) < 0) {
		allegro_message ("Error setting OpenGL graphics mode:\n%s\n"
		                 "Allegro GL error : %s\n",
		                 allegro_error, allegro_gl_error);
		return -1;
	}

  // inits FMOD and loads music files
  FMOD_System_Create(&soundsystem);
  FMOD_System_Init(soundsystem, 1, FMOD_INIT_NORMAL, NULL);
  FMOD_System_CreateSound(soundsystem, "theme.mp3", FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM | FMOD_LOOP_NORMAL, 0, &theme);
  FMOD_System_CreateSound(soundsystem, "intro.mp3", FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM | FMOD_LOOP_NORMAL, 0, &intromus);
  FMOD_System_CreateSound(soundsystem, "outro.mp3", FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM | FMOD_LOOP_NORMAL, 0, &outromus);
  // plays intro music
  FMOD_System_PlaySound(soundsystem, FMOD_CHANNEL_FREE, intromus, 0, &channel);

  // allegro installs
	install_keyboard();
	install_timer();
	install_mouse();

  // interrupt setup
	LOCK_FUNCTION(timer);
	LOCK_VARIABLE(tim);
  install_int_ex (timer, BPS_TO_TIMER(30));

  // randomizer seed
  srand(time(NULL));

  // generates world
  generate_stuff();

  // OpenGL setup
  glShadeModel (GL_SMOOTH);
	glEnable (GL_DEPTH_TEST);
	glCullFace (GL_FRONT_AND_BACK);
	glDisable (GL_CULL_FACE);
  glLineWidth(5);
  glEnable(GL_TEXTURE_2D);
  glClearColor(.4,.6,.8,.0);
  glEnable(GL_BLEND);
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_EQUAL, 1.0);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  // vertex arrays setup
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_COLOR_ARRAY);
  glVertexPointer(3, GL_FLOAT, 0, tree);
  glColorPointer(3, GL_FLOAT, 0, colors);

  // loading textures
  fence = SOIL_load_OGL_texture("fencea.tga",4,0,SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_TEXTURE_REPEATS | SOIL_FLAG_INVERT_Y );
  cloud = SOIL_load_OGL_texture("clouda.tga",4,0,SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_TEXTURE_REPEATS);
  introtex = SOIL_load_OGL_texture("introa.tga",4,0,SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_TEXTURE_REPEATS);
  outro = SOIL_load_OGL_texture("outroa.tga",4,0,SOIL_FLAG_POWER_OF_TWO | SOIL_FLAG_TEXTURE_REPEATS);

  // initialising cariables
  playerx=20.0;playerz=20.0;looky=M_PI/4;


  // main game loop
	do {
	FMOD_System_Update(soundsystem);
	  while (tim>0)
	  {

		  timer_proc();
		  tim--;
	  }
	  draw();
	} while (!key[KEY_ESC]);

 // kills fmod
   FMOD_Sound_Release(outromus);
   FMOD_System_Close(soundsystem);
   FMOD_System_Release(soundsystem);
  // done
	set_gfx_mode (GFX_TEXT, 0, 0, 0, 0);
	return 0;
}
END_OF_MAIN()

